@@ -11,7 +11,7 @@ time = datetime(2017, 9, 15, 2, 34, 0)
 # A bright auroral arc that was analyzed by Imajo et al., 2021 "Active
 # auroral arc powered by accelerated electrons from very high altitudes"
 image_time, image, ax, im = asilib.plot_fisheye(
-    asi_array_code, location_code, time, color_norm='log', force_download=False
+    asi_array_code, location_code, time, color_norm='log'
 )
 plt.colorbar(im)
 ax.axis('off')
